# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from .runs.runs import (
    RunsResource,
    AsyncRunsResource,
    RunsResourceWithRawResponse,
    AsyncRunsResourceWithRawResponse,
    RunsResourceWithStreamingResponse,
    AsyncRunsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ....types.ai import (
    mission_list_params,
    mission_create_params,
    mission_list_events_params,
    mission_update_mission_params,
)
from .mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ...._base_client import AsyncPaginator, make_request_options
from .knowledge_bases import (
    KnowledgeBasesResource,
    AsyncKnowledgeBasesResource,
    KnowledgeBasesResourceWithRawResponse,
    AsyncKnowledgeBasesResourceWithRawResponse,
    KnowledgeBasesResourceWithStreamingResponse,
    AsyncKnowledgeBasesResourceWithStreamingResponse,
)
from ....types.ai.mission_data import MissionData
from ....types.ai.mission_create_response import MissionCreateResponse
from ....types.ai.missions.runs.event_data import EventData
from ....types.ai.mission_retrieve_response import MissionRetrieveResponse
from ....types.ai.mission_update_mission_response import MissionUpdateMissionResponse

__all__ = ["MissionsResource", "AsyncMissionsResource"]


class MissionsResource(SyncAPIResource):
    @cached_property
    def runs(self) -> RunsResource:
        return RunsResource(self._client)

    @cached_property
    def knowledge_bases(self) -> KnowledgeBasesResource:
        return KnowledgeBasesResource(self._client)

    @cached_property
    def mcp_servers(self) -> McpServersResource:
        return McpServersResource(self._client)

    @cached_property
    def tools(self) -> ToolsResource:
        return ToolsResource(self._client)

    @cached_property
    def with_raw_response(self) -> MissionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MissionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MissionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MissionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        description: str | Omit = omit,
        execution_mode: Literal["external", "managed"] | Omit = omit,
        instructions: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        model: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MissionCreateResponse:
        """
        Create a new mission definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/missions",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "execution_mode": execution_mode,
                    "instructions": instructions,
                    "metadata": metadata,
                    "model": model,
                },
                mission_create_params.MissionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MissionCreateResponse,
        )

    def retrieve(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MissionRetrieveResponse:
        """
        Get a mission by ID (includes tools, knowledge_bases, mcp_servers)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MissionRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MissionData]:
        """
        List all missions for the organization

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/missions",
            page=SyncDefaultFlatPagination[MissionData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    mission_list_params.MissionListParams,
                ),
            ),
            model=MissionData,
        )

    def clone_mission(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Clone an existing mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/clone",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def delete_mission(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/missions/{mission_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list_events(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[EventData]:
        """
        List recent events across all missions

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/missions/events",
            page=SyncDefaultFlatPagination[EventData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "type": type,
                    },
                    mission_list_events_params.MissionListEventsParams,
                ),
            ),
            model=EventData,
        )

    def update_mission(
        self,
        mission_id: str,
        *,
        description: str | Omit = omit,
        execution_mode: Literal["external", "managed"] | Omit = omit,
        instructions: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        model: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MissionUpdateMissionResponse:
        """
        Update a mission definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return self._put(
            f"/ai/missions/{mission_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "execution_mode": execution_mode,
                    "instructions": instructions,
                    "metadata": metadata,
                    "model": model,
                    "name": name,
                },
                mission_update_mission_params.MissionUpdateMissionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MissionUpdateMissionResponse,
        )


class AsyncMissionsResource(AsyncAPIResource):
    @cached_property
    def runs(self) -> AsyncRunsResource:
        return AsyncRunsResource(self._client)

    @cached_property
    def knowledge_bases(self) -> AsyncKnowledgeBasesResource:
        return AsyncKnowledgeBasesResource(self._client)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResource:
        return AsyncMcpServersResource(self._client)

    @cached_property
    def tools(self) -> AsyncToolsResource:
        return AsyncToolsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMissionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMissionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMissionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMissionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        description: str | Omit = omit,
        execution_mode: Literal["external", "managed"] | Omit = omit,
        instructions: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        model: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MissionCreateResponse:
        """
        Create a new mission definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/missions",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "execution_mode": execution_mode,
                    "instructions": instructions,
                    "metadata": metadata,
                    "model": model,
                },
                mission_create_params.MissionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MissionCreateResponse,
        )

    async def retrieve(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MissionRetrieveResponse:
        """
        Get a mission by ID (includes tools, knowledge_bases, mcp_servers)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MissionRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MissionData, AsyncDefaultFlatPagination[MissionData]]:
        """
        List all missions for the organization

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/missions",
            page=AsyncDefaultFlatPagination[MissionData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    mission_list_params.MissionListParams,
                ),
            ),
            model=MissionData,
        )

    async def clone_mission(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Clone an existing mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/clone",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def delete_mission(
        self,
        mission_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a mission

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/missions/{mission_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list_events(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[EventData, AsyncDefaultFlatPagination[EventData]]:
        """
        List recent events across all missions

        Args:
          page_number: Page number (1-based)

          page_size: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/missions/events",
            page=AsyncDefaultFlatPagination[EventData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "type": type,
                    },
                    mission_list_events_params.MissionListEventsParams,
                ),
            ),
            model=EventData,
        )

    async def update_mission(
        self,
        mission_id: str,
        *,
        description: str | Omit = omit,
        execution_mode: Literal["external", "managed"] | Omit = omit,
        instructions: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        model: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MissionUpdateMissionResponse:
        """
        Update a mission definition

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        return await self._put(
            f"/ai/missions/{mission_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "execution_mode": execution_mode,
                    "instructions": instructions,
                    "metadata": metadata,
                    "model": model,
                    "name": name,
                },
                mission_update_mission_params.MissionUpdateMissionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MissionUpdateMissionResponse,
        )


class MissionsResourceWithRawResponse:
    def __init__(self, missions: MissionsResource) -> None:
        self._missions = missions

        self.create = to_raw_response_wrapper(
            missions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            missions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            missions.list,
        )
        self.clone_mission = to_raw_response_wrapper(
            missions.clone_mission,
        )
        self.delete_mission = to_raw_response_wrapper(
            missions.delete_mission,
        )
        self.list_events = to_raw_response_wrapper(
            missions.list_events,
        )
        self.update_mission = to_raw_response_wrapper(
            missions.update_mission,
        )

    @cached_property
    def runs(self) -> RunsResourceWithRawResponse:
        return RunsResourceWithRawResponse(self._missions.runs)

    @cached_property
    def knowledge_bases(self) -> KnowledgeBasesResourceWithRawResponse:
        return KnowledgeBasesResourceWithRawResponse(self._missions.knowledge_bases)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithRawResponse:
        return McpServersResourceWithRawResponse(self._missions.mcp_servers)

    @cached_property
    def tools(self) -> ToolsResourceWithRawResponse:
        return ToolsResourceWithRawResponse(self._missions.tools)


class AsyncMissionsResourceWithRawResponse:
    def __init__(self, missions: AsyncMissionsResource) -> None:
        self._missions = missions

        self.create = async_to_raw_response_wrapper(
            missions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            missions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            missions.list,
        )
        self.clone_mission = async_to_raw_response_wrapper(
            missions.clone_mission,
        )
        self.delete_mission = async_to_raw_response_wrapper(
            missions.delete_mission,
        )
        self.list_events = async_to_raw_response_wrapper(
            missions.list_events,
        )
        self.update_mission = async_to_raw_response_wrapper(
            missions.update_mission,
        )

    @cached_property
    def runs(self) -> AsyncRunsResourceWithRawResponse:
        return AsyncRunsResourceWithRawResponse(self._missions.runs)

    @cached_property
    def knowledge_bases(self) -> AsyncKnowledgeBasesResourceWithRawResponse:
        return AsyncKnowledgeBasesResourceWithRawResponse(self._missions.knowledge_bases)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithRawResponse:
        return AsyncMcpServersResourceWithRawResponse(self._missions.mcp_servers)

    @cached_property
    def tools(self) -> AsyncToolsResourceWithRawResponse:
        return AsyncToolsResourceWithRawResponse(self._missions.tools)


class MissionsResourceWithStreamingResponse:
    def __init__(self, missions: MissionsResource) -> None:
        self._missions = missions

        self.create = to_streamed_response_wrapper(
            missions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            missions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            missions.list,
        )
        self.clone_mission = to_streamed_response_wrapper(
            missions.clone_mission,
        )
        self.delete_mission = to_streamed_response_wrapper(
            missions.delete_mission,
        )
        self.list_events = to_streamed_response_wrapper(
            missions.list_events,
        )
        self.update_mission = to_streamed_response_wrapper(
            missions.update_mission,
        )

    @cached_property
    def runs(self) -> RunsResourceWithStreamingResponse:
        return RunsResourceWithStreamingResponse(self._missions.runs)

    @cached_property
    def knowledge_bases(self) -> KnowledgeBasesResourceWithStreamingResponse:
        return KnowledgeBasesResourceWithStreamingResponse(self._missions.knowledge_bases)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithStreamingResponse:
        return McpServersResourceWithStreamingResponse(self._missions.mcp_servers)

    @cached_property
    def tools(self) -> ToolsResourceWithStreamingResponse:
        return ToolsResourceWithStreamingResponse(self._missions.tools)


class AsyncMissionsResourceWithStreamingResponse:
    def __init__(self, missions: AsyncMissionsResource) -> None:
        self._missions = missions

        self.create = async_to_streamed_response_wrapper(
            missions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            missions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            missions.list,
        )
        self.clone_mission = async_to_streamed_response_wrapper(
            missions.clone_mission,
        )
        self.delete_mission = async_to_streamed_response_wrapper(
            missions.delete_mission,
        )
        self.list_events = async_to_streamed_response_wrapper(
            missions.list_events,
        )
        self.update_mission = async_to_streamed_response_wrapper(
            missions.update_mission,
        )

    @cached_property
    def runs(self) -> AsyncRunsResourceWithStreamingResponse:
        return AsyncRunsResourceWithStreamingResponse(self._missions.runs)

    @cached_property
    def knowledge_bases(self) -> AsyncKnowledgeBasesResourceWithStreamingResponse:
        return AsyncKnowledgeBasesResourceWithStreamingResponse(self._missions.knowledge_bases)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithStreamingResponse:
        return AsyncMcpServersResourceWithStreamingResponse(self._missions.mcp_servers)

    @cached_property
    def tools(self) -> AsyncToolsResourceWithStreamingResponse:
        return AsyncToolsResourceWithStreamingResponse(self._missions.tools)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .civic_address import CivicAddress

__all__ = ["CivicAddressRetrieveResponse"]


class CivicAddressRetrieveResponse(BaseModel):
    data: Optional[CivicAddress] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.ai import (
    McpServerListResponse,
    McpServerCreateResponse,
    McpServerUpdateResponse,
    McpServerRetrieveResponse,
)
from telnyx.pagination import SyncDefaultFlatPaginationTopLevelArray, AsyncDefaultFlatPaginationTopLevelArray

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMcpServers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.create(
            name="name",
            type="type",
            url="url",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.create(
            name="name",
            type="type",
            url="url",
            allowed_tools=["string"],
            api_key_ref="api_key_ref",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.ai.mcp_servers.with_raw_response.create(
            name="name",
            type="type",
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.ai.mcp_servers.with_streaming_response.create(
            name="name",
            type="type",
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.retrieve(
            "mcp_server_id",
        )
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ai.mcp_servers.with_raw_response.retrieve(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ai.mcp_servers.with_streaming_response.retrieve(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            client.ai.mcp_servers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.update(
            mcp_server_id="mcp_server_id",
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.update(
            mcp_server_id="mcp_server_id",
            id="id",
            allowed_tools=["string"],
            api_key_ref="api_key_ref",
            created_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            name="name",
            type="type",
            url="url",
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.ai.mcp_servers.with_raw_response.update(
            mcp_server_id="mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.ai.mcp_servers.with_streaming_response.update(
            mcp_server_id="mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            client.ai.mcp_servers.with_raw_response.update(
                mcp_server_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.list()
        assert_matches_type(
            SyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.list(
            page_number=1,
            page_size=1,
            type="type",
            url="url",
        )
        assert_matches_type(
            SyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ai.mcp_servers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(
            SyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ai.mcp_servers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(
                SyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        mcp_server = client.ai.mcp_servers.delete(
            "mcp_server_id",
        )
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.ai.mcp_servers.with_raw_response.delete(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.ai.mcp_servers.with_streaming_response.delete(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert mcp_server is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            client.ai.mcp_servers.with_raw_response.delete(
                "",
            )


class TestAsyncMcpServers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.create(
            name="name",
            type="type",
            url="url",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.create(
            name="name",
            type="type",
            url="url",
            allowed_tools=["string"],
            api_key_ref="api_key_ref",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.mcp_servers.with_raw_response.create(
            name="name",
            type="type",
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.mcp_servers.with_streaming_response.create(
            name="name",
            type="type",
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.retrieve(
            "mcp_server_id",
        )
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.mcp_servers.with_raw_response.retrieve(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.mcp_servers.with_streaming_response.retrieve(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            await async_client.ai.mcp_servers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.update(
            mcp_server_id="mcp_server_id",
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.update(
            mcp_server_id="mcp_server_id",
            id="id",
            allowed_tools=["string"],
            api_key_ref="api_key_ref",
            created_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            name="name",
            type="type",
            url="url",
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.mcp_servers.with_raw_response.update(
            mcp_server_id="mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.mcp_servers.with_streaming_response.update(
            mcp_server_id="mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            await async_client.ai.mcp_servers.with_raw_response.update(
                mcp_server_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.list()
        assert_matches_type(
            AsyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.list(
            page_number=1,
            page_size=1,
            type="type",
            url="url",
        )
        assert_matches_type(
            AsyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.mcp_servers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.mcp_servers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPaginationTopLevelArray[McpServerListResponse], mcp_server, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        mcp_server = await async_client.ai.mcp_servers.delete(
            "mcp_server_id",
        )
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.mcp_servers.with_raw_response.delete(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.mcp_servers.with_streaming_response.delete(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert mcp_server is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            await async_client.ai.mcp_servers.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CallControlCommandResultWithConversationID"]


class CallControlCommandResultWithConversationID(BaseModel):
    conversation_id: Optional[str] = None
    """The ID of the conversation created by the command."""

    result: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.phone_numbers import (
    voice_list_params,
    voice_update_params,
)
from ...types.phone_numbers.cnam_listing_param import CnamListingParam
from ...types.phone_numbers.call_recording_param import CallRecordingParam
from ...types.phone_numbers.media_features_param import MediaFeaturesParam
from ...types.phone_numbers.call_forwarding_param import CallForwardingParam
from ...types.phone_numbers.voice_update_response import VoiceUpdateResponse
from ...types.phone_numbers.voice_retrieve_response import VoiceRetrieveResponse
from ...types.phone_numbers.phone_number_with_voice_settings import PhoneNumberWithVoiceSettings

__all__ = ["VoiceResource", "AsyncVoiceResource"]


class VoiceResource(SyncAPIResource):
    """Configure your phone numbers"""

    @cached_property
    def with_raw_response(self) -> VoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VoiceResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveResponse:
        """
        Retrieve a phone number with voice settings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/phone_numbers/{id}/voice",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        call_forwarding: CallForwardingParam | Omit = omit,
        call_recording: CallRecordingParam | Omit = omit,
        caller_id_name_enabled: bool | Omit = omit,
        cnam_listing: CnamListingParam | Omit = omit,
        inbound_call_screening: Literal["disabled", "reject_calls", "flag_calls"] | Omit = omit,
        media_features: MediaFeaturesParam | Omit = omit,
        tech_prefix_enabled: bool | Omit = omit,
        translated_number: str | Omit = omit,
        usage_payment_method: Literal["pay-per-minute", "channel"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceUpdateResponse:
        """
        Update a phone number with voice settings

        Args:
          call_forwarding: The call forwarding settings for a phone number.

          call_recording: The call recording settings for a phone number.

          caller_id_name_enabled: Controls whether the caller ID name is enabled for this phone number.

          cnam_listing: The CNAM listing settings for a phone number.

          inbound_call_screening: The inbound_call_screening setting is a phone number configuration option
              variable that allows users to configure their settings to block or flag
              fraudulent calls. It can be set to disabled, reject_calls, or flag_calls. This
              feature has an additional per-number monthly cost associated with it.

          media_features: The media features settings for a phone number.

          tech_prefix_enabled: Controls whether a tech prefix is enabled for this phone number.

          translated_number: This field allows you to rewrite the destination number of an inbound call
              before the call is routed to you. The value of this field may be any
              alphanumeric value, and the value will replace the number originally dialed.

          usage_payment_method: Controls whether a number is billed per minute or uses your concurrent channels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/phone_numbers/{id}/voice",
            body=maybe_transform(
                {
                    "call_forwarding": call_forwarding,
                    "call_recording": call_recording,
                    "caller_id_name_enabled": caller_id_name_enabled,
                    "cnam_listing": cnam_listing,
                    "inbound_call_screening": inbound_call_screening,
                    "media_features": media_features,
                    "tech_prefix_enabled": tech_prefix_enabled,
                    "translated_number": translated_number,
                    "usage_payment_method": usage_payment_method,
                },
                voice_update_params.VoiceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceUpdateResponse,
        )

    def list(
        self,
        *,
        filter: voice_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["purchased_at", "phone_number", "connection_name", "usage_payment_method"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PhoneNumberWithVoiceSettings]:
        """
        List phone numbers with voice settings

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[phone_number], filter[connection_name], filter[customer_reference],
              filter[voice.usage_payment_method]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/phone_numbers/voice",
            page=SyncDefaultFlatPagination[PhoneNumberWithVoiceSettings],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    voice_list_params.VoiceListParams,
                ),
            ),
            model=PhoneNumberWithVoiceSettings,
        )


class AsyncVoiceResource(AsyncAPIResource):
    """Configure your phone numbers"""

    @cached_property
    def with_raw_response(self) -> AsyncVoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVoiceResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceRetrieveResponse:
        """
        Retrieve a phone number with voice settings

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/phone_numbers/{id}/voice",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        call_forwarding: CallForwardingParam | Omit = omit,
        call_recording: CallRecordingParam | Omit = omit,
        caller_id_name_enabled: bool | Omit = omit,
        cnam_listing: CnamListingParam | Omit = omit,
        inbound_call_screening: Literal["disabled", "reject_calls", "flag_calls"] | Omit = omit,
        media_features: MediaFeaturesParam | Omit = omit,
        tech_prefix_enabled: bool | Omit = omit,
        translated_number: str | Omit = omit,
        usage_payment_method: Literal["pay-per-minute", "channel"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceUpdateResponse:
        """
        Update a phone number with voice settings

        Args:
          call_forwarding: The call forwarding settings for a phone number.

          call_recording: The call recording settings for a phone number.

          caller_id_name_enabled: Controls whether the caller ID name is enabled for this phone number.

          cnam_listing: The CNAM listing settings for a phone number.

          inbound_call_screening: The inbound_call_screening setting is a phone number configuration option
              variable that allows users to configure their settings to block or flag
              fraudulent calls. It can be set to disabled, reject_calls, or flag_calls. This
              feature has an additional per-number monthly cost associated with it.

          media_features: The media features settings for a phone number.

          tech_prefix_enabled: Controls whether a tech prefix is enabled for this phone number.

          translated_number: This field allows you to rewrite the destination number of an inbound call
              before the call is routed to you. The value of this field may be any
              alphanumeric value, and the value will replace the number originally dialed.

          usage_payment_method: Controls whether a number is billed per minute or uses your concurrent channels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/phone_numbers/{id}/voice",
            body=await async_maybe_transform(
                {
                    "call_forwarding": call_forwarding,
                    "call_recording": call_recording,
                    "caller_id_name_enabled": caller_id_name_enabled,
                    "cnam_listing": cnam_listing,
                    "inbound_call_screening": inbound_call_screening,
                    "media_features": media_features,
                    "tech_prefix_enabled": tech_prefix_enabled,
                    "translated_number": translated_number,
                    "usage_payment_method": usage_payment_method,
                },
                voice_update_params.VoiceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceUpdateResponse,
        )

    def list(
        self,
        *,
        filter: voice_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["purchased_at", "phone_number", "connection_name", "usage_payment_method"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PhoneNumberWithVoiceSettings, AsyncDefaultFlatPagination[PhoneNumberWithVoiceSettings]]:
        """
        List phone numbers with voice settings

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[phone_number], filter[connection_name], filter[customer_reference],
              filter[voice.usage_payment_method]

          sort: Specifies the sort order for results. If not given, results are sorted by
              created_at in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/phone_numbers/voice",
            page=AsyncDefaultFlatPagination[PhoneNumberWithVoiceSettings],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    voice_list_params.VoiceListParams,
                ),
            ),
            model=PhoneNumberWithVoiceSettings,
        )


class VoiceResourceWithRawResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.retrieve = to_raw_response_wrapper(
            voice.retrieve,
        )
        self.update = to_raw_response_wrapper(
            voice.update,
        )
        self.list = to_raw_response_wrapper(
            voice.list,
        )


class AsyncVoiceResourceWithRawResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.retrieve = async_to_raw_response_wrapper(
            voice.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            voice.update,
        )
        self.list = async_to_raw_response_wrapper(
            voice.list,
        )


class VoiceResourceWithStreamingResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.retrieve = to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            voice.update,
        )
        self.list = to_streamed_response_wrapper(
            voice.list,
        )


class AsyncVoiceResourceWithStreamingResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.retrieve = async_to_streamed_response_wrapper(
            voice.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            voice.update,
        )
        self.list = async_to_streamed_response_wrapper(
            voice.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .json_retrieve_recording_transcription_sid_json_response import (
    JsonRetrieveRecordingTranscriptionSidJsonResponse as JsonRetrieveRecordingTranscriptionSidJsonResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import messaging_optout_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.messaging_optout_list_response import MessagingOptoutListResponse

__all__ = ["MessagingOptoutsResource", "AsyncMessagingOptoutsResource"]


class MessagingOptoutsResource(SyncAPIResource):
    """Opt-Out Management"""

    @cached_property
    def with_raw_response(self) -> MessagingOptoutsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagingOptoutsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagingOptoutsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagingOptoutsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        created_at: messaging_optout_list_params.CreatedAt | Omit = omit,
        filter: messaging_optout_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        redaction_enabled: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MessagingOptoutListResponse]:
        """
        Retrieve a list of opt-out blocks.

        Args:
          created_at:
              Consolidated created_at parameter (deepObject style). Originally:
              created_at[gte], created_at[lte]

          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[messaging_profile_id], filter[from]

          redaction_enabled: If receiving address (+E.164 formatted phone number) should be redacted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/messaging_optouts",
            page=SyncDefaultFlatPagination[MessagingOptoutListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at": created_at,
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "redaction_enabled": redaction_enabled,
                    },
                    messaging_optout_list_params.MessagingOptoutListParams,
                ),
            ),
            model=MessagingOptoutListResponse,
        )


class AsyncMessagingOptoutsResource(AsyncAPIResource):
    """Opt-Out Management"""

    @cached_property
    def with_raw_response(self) -> AsyncMessagingOptoutsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagingOptoutsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagingOptoutsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagingOptoutsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        created_at: messaging_optout_list_params.CreatedAt | Omit = omit,
        filter: messaging_optout_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        redaction_enabled: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MessagingOptoutListResponse, AsyncDefaultFlatPagination[MessagingOptoutListResponse]]:
        """
        Retrieve a list of opt-out blocks.

        Args:
          created_at:
              Consolidated created_at parameter (deepObject style). Originally:
              created_at[gte], created_at[lte]

          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[messaging_profile_id], filter[from]

          redaction_enabled: If receiving address (+E.164 formatted phone number) should be redacted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/messaging_optouts",
            page=AsyncDefaultFlatPagination[MessagingOptoutListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at": created_at,
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "redaction_enabled": redaction_enabled,
                    },
                    messaging_optout_list_params.MessagingOptoutListParams,
                ),
            ),
            model=MessagingOptoutListResponse,
        )


class MessagingOptoutsResourceWithRawResponse:
    def __init__(self, messaging_optouts: MessagingOptoutsResource) -> None:
        self._messaging_optouts = messaging_optouts

        self.list = to_raw_response_wrapper(
            messaging_optouts.list,
        )


class AsyncMessagingOptoutsResourceWithRawResponse:
    def __init__(self, messaging_optouts: AsyncMessagingOptoutsResource) -> None:
        self._messaging_optouts = messaging_optouts

        self.list = async_to_raw_response_wrapper(
            messaging_optouts.list,
        )


class MessagingOptoutsResourceWithStreamingResponse:
    def __init__(self, messaging_optouts: MessagingOptoutsResource) -> None:
        self._messaging_optouts = messaging_optouts

        self.list = to_streamed_response_wrapper(
            messaging_optouts.list,
        )


class AsyncMessagingOptoutsResourceWithStreamingResponse:
    def __init__(self, messaging_optouts: AsyncMessagingOptoutsResource) -> None:
        self._messaging_optouts = messaging_optouts

        self.list = async_to_streamed_response_wrapper(
            messaging_optouts.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SimCardAction", "Status"]


class Status(BaseModel):
    reason: Optional[str] = None
    """It describes why the SIM card action is in the current status.

    This will be <code>null</code> for self-explanatory statuses, such as
    <code>in-progress</code> and <code>completed</code> but will include further
    information on statuses like <code>interrupted</code> and <code>failed</code>.
    """

    value: Optional[Literal["in-progress", "completed", "failed", "interrupted"]] = None
    """The current status of the SIM card action."""


class SimCardAction(BaseModel):
    """This object represents a SIM card action.

    It allows tracking the current status of an operation that impacts the SIM card.
    """

    id: Optional[str] = None
    """Identifies the resource."""

    action_type: Optional[Literal["enable", "enable_standby_sim_card", "disable", "set_standby"]] = None
    """The operation type. It can be one of the following: <br/>

    <ul>
     <li><code>enable</code> - move the SIM card to the <code>enabled</code> status</li>
     <li><code>enable_standby_sim_card</code> - move a SIM card previously on the <code>standby</code> status to the <code>enabled</code> status after it consumes data.</li>
     <li><code>disable</code> - move the SIM card to the <code>disabled</code> status</li>
     <li><code>set_standby</code> - move the SIM card to the <code>standby</code> status</li>
     </ul>
    """

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    record_type: Optional[str] = None

    settings: Optional[Dict[str, object]] = None
    """A JSON object representation of the action params."""

    sim_card_id: Optional[str] = None
    """The related SIM card identifier."""

    status: Optional[Status] = None

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel
from .verification import Verification

__all__ = ["CreateVerificationResponse"]


class CreateVerificationResponse(BaseModel):
    data: Verification



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .waba_settings import WabaSettings as WabaSettings
from .setting_update_params import SettingUpdateParams as SettingUpdateParams
from .setting_update_response import SettingUpdateResponse as SettingUpdateResponse
from .phone_number_list_params import PhoneNumberListParams as PhoneNumberListParams
from .setting_retrieve_response import SettingRetrieveResponse as SettingRetrieveResponse
from .phone_number_list_response import PhoneNumberListResponse as PhoneNumberListResponse
from .phone_number_initialize_verification_params import (
    PhoneNumberInitializeVerificationParams as PhoneNumberInitializeVerificationParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    VerifyProfile,
    MessageTemplate,
    VerifyProfileData,
    VerifyProfileRetrieveTemplatesResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVerifyProfiles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.create(
            name="Test Profile",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.create(
            name="Test Profile",
            call={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            flashcall={
                "app_name": "Example Secure App",
                "default_verification_timeout_secs": 300,
                "whitelisted_destinations": ["US", "CA"],
            },
            language="en-US",
            rcs={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "sms_fallback": True,
                "whitelisted_destinations": ["US", "CA"],
            },
            sms={
                "alpha_sender": "sqF",
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            webhook_failover_url="http://example.com/webhook/failover",
            webhook_url="http://example.com/webhook",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.create(
            name="Test Profile",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.create(
            name="Test Profile",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verify_profile_id` but received ''"):
            client.verify_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            call={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            flashcall={
                "app_name": "Example Secure App",
                "default_verification_timeout_secs": 300,
                "whitelisted_destinations": ["US", "CA"],
            },
            language="en-US",
            name="Test Profile",
            rcs={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "sms_fallback": True,
                "whitelisted_destinations": ["US", "CA"],
            },
            sms={
                "alpha_sender": "sqF",
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            webhook_failover_url="http://example.com/webhook/failover",
            webhook_url="http://example.com/webhook",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verify_profile_id` but received ''"):
            client.verify_profiles.with_raw_response.update(
                verify_profile_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.list()
        assert_matches_type(SyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.list(
            filter={"name": "name"},
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.delete(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.delete(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.delete(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verify_profile_id` but received ''"):
            client.verify_profiles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_template(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.create_template(
            text="Your {{app_name}} verification code is: {{code}}.",
        )
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_template(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.create_template(
            text="Your {{app_name}} verification code is: {{code}}.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_template(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.create_template(
            text="Your {{app_name}} verification code is: {{code}}.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(MessageTemplate, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_templates(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.retrieve_templates()
        assert_matches_type(VerifyProfileRetrieveTemplatesResponse, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve_templates(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.retrieve_templates()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(VerifyProfileRetrieveTemplatesResponse, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_templates(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.retrieve_templates() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(VerifyProfileRetrieveTemplatesResponse, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_template(self, client: Telnyx) -> None:
        verify_profile = client.verify_profiles.update_template(
            template_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            text="Your {{app_name}} verification code is: {{code}}.",
        )
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_template(self, client: Telnyx) -> None:
        response = client.verify_profiles.with_raw_response.update_template(
            template_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            text="Your {{app_name}} verification code is: {{code}}.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = response.parse()
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_template(self, client: Telnyx) -> None:
        with client.verify_profiles.with_streaming_response.update_template(
            template_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            text="Your {{app_name}} verification code is: {{code}}.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = response.parse()
            assert_matches_type(MessageTemplate, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_template(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            client.verify_profiles.with_raw_response.update_template(
                template_id="",
                text="Your {{app_name}} verification code is: {{code}}.",
            )


class TestAsyncVerifyProfiles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.create(
            name="Test Profile",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.create(
            name="Test Profile",
            call={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            flashcall={
                "app_name": "Example Secure App",
                "default_verification_timeout_secs": 300,
                "whitelisted_destinations": ["US", "CA"],
            },
            language="en-US",
            rcs={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "sms_fallback": True,
                "whitelisted_destinations": ["US", "CA"],
            },
            sms={
                "alpha_sender": "sqF",
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            webhook_failover_url="http://example.com/webhook/failover",
            webhook_url="http://example.com/webhook",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.create(
            name="Test Profile",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.create(
            name="Test Profile",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verify_profile_id` but received ''"):
            await async_client.verify_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            call={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            flashcall={
                "app_name": "Example Secure App",
                "default_verification_timeout_secs": 300,
                "whitelisted_destinations": ["US", "CA"],
            },
            language="en-US",
            name="Test Profile",
            rcs={
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "sms_fallback": True,
                "whitelisted_destinations": ["US", "CA"],
            },
            sms={
                "alpha_sender": "sqF",
                "app_name": "Example Secure App",
                "code_length": 6,
                "default_verification_timeout_secs": 300,
                "messaging_template_id": "0abb5b4f-459f-445a-bfcd-488998b7572d",
                "whitelisted_destinations": ["US", "CA"],
            },
            webhook_failover_url="http://example.com/webhook/failover",
            webhook_url="http://example.com/webhook",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.update(
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verify_profile_id` but received ''"):
            await async_client.verify_profiles.with_raw_response.update(
                verify_profile_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.list()
        assert_matches_type(AsyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.list(
            filter={"name": "name"},
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[VerifyProfile], verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.delete(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.delete(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.delete(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(VerifyProfileData, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verify_profile_id` but received ''"):
            await async_client.verify_profiles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_template(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.create_template(
            text="Your {{app_name}} verification code is: {{code}}.",
        )
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_template(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.create_template(
            text="Your {{app_name}} verification code is: {{code}}.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_template(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.create_template(
            text="Your {{app_name}} verification code is: {{code}}.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(MessageTemplate, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_templates(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.retrieve_templates()
        assert_matches_type(VerifyProfileRetrieveTemplatesResponse, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_templates(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.retrieve_templates()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(VerifyProfileRetrieveTemplatesResponse, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_templates(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.retrieve_templates() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(VerifyProfileRetrieveTemplatesResponse, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_template(self, async_client: AsyncTelnyx) -> None:
        verify_profile = await async_client.verify_profiles.update_template(
            template_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            text="Your {{app_name}} verification code is: {{code}}.",
        )
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_template(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verify_profiles.with_raw_response.update_template(
            template_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            text="Your {{app_name}} verification code is: {{code}}.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verify_profile = await response.parse()
        assert_matches_type(MessageTemplate, verify_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_template(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verify_profiles.with_streaming_response.update_template(
            template_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            text="Your {{app_name}} verification code is: {{code}}.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verify_profile = await response.parse()
            assert_matches_type(MessageTemplate, verify_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_template(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            await async_client.verify_profiles.with_raw_response.update_template(
                template_id="",
                text="Your {{app_name}} verification code is: {{code}}.",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ShortCode"]


class ShortCode(BaseModel):
    messaging_profile_id: Optional[str] = None
    """Unique identifier for a messaging profile."""

    id: Optional[str] = None
    """Identifies the type of resource."""

    country_code: Optional[str] = None
    """ISO 3166-1 alpha-2 country code."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    record_type: Optional[Literal["short_code"]] = None
    """Identifies the type of the resource."""

    short_code: Optional[str] = None
    """Short digit sequence used to address messages."""

    tags: Optional[List[str]] = None

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .job import Job
from ..._models import BaseModel

__all__ = ["JobRetrieveResponse"]


class JobRetrieveResponse(BaseModel):
    data: Optional[Job] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .number_order_phone_number import NumberOrderPhoneNumber

__all__ = ["NumberOrderPhoneNumberRetrieveResponse"]


class NumberOrderPhoneNumberRetrieveResponse(BaseModel):
    data: Optional[NumberOrderPhoneNumber] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .porting_event_split_event import PortingEventSplitEvent
from .porting_event_deleted_payload import PortingEventDeletedPayload
from .porting_event_without_webhook import PortingEventWithoutWebhook
from .porting_event_new_comment_event import PortingEventNewCommentEvent
from .porting_event_status_changed_event import PortingEventStatusChangedEvent
from .porting_event_messaging_changed_payload import PortingEventMessagingChangedPayload

__all__ = ["EventRetrieveResponse", "Data"]

Data: TypeAlias = Annotated[
    Union[
        PortingEventDeletedPayload,
        PortingEventMessagingChangedPayload,
        PortingEventStatusChangedEvent,
        PortingEventNewCommentEvent,
        PortingEventSplitEvent,
        PortingEventWithoutWebhook,
    ],
    PropertyInfo(discriminator="event_type"),
]


class EventRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo
from ..filter_param import FilterParam

__all__ = ["VoiceCreateParams"]


class VoiceCreateParams(TypedDict, total=False):
    end_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """End time in ISO format"""

    start_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Start time in ISO format"""

    call_types: Iterable[int]
    """List of call types to filter by (Inbound = 1, Outbound = 2)"""

    connections: Iterable[int]
    """List of connections to filter by"""

    fields: SequenceNotStr[str]
    """Set of fields to include in the report"""

    filters: Iterable[FilterParam]
    """List of filters to apply"""

    include_all_metadata: bool
    """Whether to include all metadata"""

    managed_accounts: SequenceNotStr[str]
    """List of managed accounts to include"""

    record_types: Iterable[int]
    """List of record types to filter by (Complete = 1, Incomplete = 2, Errors = 3)"""

    report_name: str
    """Name of the report"""

    select_all_managed_accounts: bool
    """Whether to select all managed accounts"""

    source: str
    """Source of the report.

    Valid values: calls (default), call-control, fax-api, webrtc
    """

    timezone: str
    """Timezone for the report"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, TypeAlias, TypedDict

from .deepgram_nova2_config_param import DeepgramNova2ConfigParam
from .deepgram_nova3_config_param import DeepgramNova3ConfigParam
from .transcription_engine_a_config_param import TranscriptionEngineAConfigParam
from .transcription_engine_b_config_param import TranscriptionEngineBConfigParam
from .transcription_engine_azure_config_param import TranscriptionEngineAzureConfigParam
from .transcription_engine_google_config_param import TranscriptionEngineGoogleConfigParam
from .transcription_engine_telnyx_config_param import TranscriptionEngineTelnyxConfigParam

__all__ = ["ActionStartTranscriptionParams", "TranscriptionEngineConfig"]


class ActionStartTranscriptionParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    transcription_engine: Literal["Google", "Telnyx", "Deepgram", "Azure", "A", "B"]
    """Engine to use for speech recognition.

    Legacy values `A` - `Google`, `B` - `Telnyx` are supported for backward
    compatibility.
    """

    transcription_engine_config: TranscriptionEngineConfig

    transcription_tracks: str
    """Indicates which leg of the call will be transcribed.

    Use `inbound` for the leg that requested the transcription, `outbound` for the
    other leg, and `both` for both legs of the call. Will default to `inbound`.
    """


TranscriptionEngineConfig: TypeAlias = Union[
    TranscriptionEngineGoogleConfigParam,
    TranscriptionEngineTelnyxConfigParam,
    TranscriptionEngineAzureConfigParam,
    TranscriptionEngineAConfigParam,
    TranscriptionEngineBConfigParam,
    DeepgramNova2ConfigParam,
    DeepgramNova3ConfigParam,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from .templates import (
    TemplatesResource,
    AsyncTemplatesResource,
    TemplatesResourceWithRawResponse,
    AsyncTemplatesResourceWithRawResponse,
    TemplatesResourceWithStreamingResponse,
    AsyncTemplatesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .phone_numbers.phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)
from .business_accounts.business_accounts import (
    BusinessAccountsResource,
    AsyncBusinessAccountsResource,
    BusinessAccountsResourceWithRawResponse,
    AsyncBusinessAccountsResourceWithRawResponse,
    BusinessAccountsResourceWithStreamingResponse,
    AsyncBusinessAccountsResourceWithStreamingResponse,
)

__all__ = ["WhatsappResource", "AsyncWhatsappResource"]


class WhatsappResource(SyncAPIResource):
    @cached_property
    def business_accounts(self) -> BusinessAccountsResource:
        """Manage Whatsapp business accounts"""
        return BusinessAccountsResource(self._client)

    @cached_property
    def templates(self) -> TemplatesResource:
        """Manage Whatsapp message templates"""
        return TemplatesResource(self._client)

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResource:
        """Manage Whatsapp phone numbers"""
        return PhoneNumbersResource(self._client)

    @cached_property
    def with_raw_response(self) -> WhatsappResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return WhatsappResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WhatsappResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return WhatsappResourceWithStreamingResponse(self)


class AsyncWhatsappResource(AsyncAPIResource):
    @cached_property
    def business_accounts(self) -> AsyncBusinessAccountsResource:
        """Manage Whatsapp business accounts"""
        return AsyncBusinessAccountsResource(self._client)

    @cached_property
    def templates(self) -> AsyncTemplatesResource:
        """Manage Whatsapp message templates"""
        return AsyncTemplatesResource(self._client)

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResource:
        """Manage Whatsapp phone numbers"""
        return AsyncPhoneNumbersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWhatsappResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWhatsappResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWhatsappResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncWhatsappResourceWithStreamingResponse(self)


class WhatsappResourceWithRawResponse:
    def __init__(self, whatsapp: WhatsappResource) -> None:
        self._whatsapp = whatsapp

    @cached_property
    def business_accounts(self) -> BusinessAccountsResourceWithRawResponse:
        """Manage Whatsapp business accounts"""
        return BusinessAccountsResourceWithRawResponse(self._whatsapp.business_accounts)

    @cached_property
    def templates(self) -> TemplatesResourceWithRawResponse:
        """Manage Whatsapp message templates"""
        return TemplatesResourceWithRawResponse(self._whatsapp.templates)

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return PhoneNumbersResourceWithRawResponse(self._whatsapp.phone_numbers)


class AsyncWhatsappResourceWithRawResponse:
    def __init__(self, whatsapp: AsyncWhatsappResource) -> None:
        self._whatsapp = whatsapp

    @cached_property
    def business_accounts(self) -> AsyncBusinessAccountsResourceWithRawResponse:
        """Manage Whatsapp business accounts"""
        return AsyncBusinessAccountsResourceWithRawResponse(self._whatsapp.business_accounts)

    @cached_property
    def templates(self) -> AsyncTemplatesResourceWithRawResponse:
        """Manage Whatsapp message templates"""
        return AsyncTemplatesResourceWithRawResponse(self._whatsapp.templates)

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncPhoneNumbersResourceWithRawResponse(self._whatsapp.phone_numbers)


class WhatsappResourceWithStreamingResponse:
    def __init__(self, whatsapp: WhatsappResource) -> None:
        self._whatsapp = whatsapp

    @cached_property
    def business_accounts(self) -> BusinessAccountsResourceWithStreamingResponse:
        """Manage Whatsapp business accounts"""
        return BusinessAccountsResourceWithStreamingResponse(self._whatsapp.business_accounts)

    @cached_property
    def templates(self) -> TemplatesResourceWithStreamingResponse:
        """Manage Whatsapp message templates"""
        return TemplatesResourceWithStreamingResponse(self._whatsapp.templates)

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return PhoneNumbersResourceWithStreamingResponse(self._whatsapp.phone_numbers)


class AsyncWhatsappResourceWithStreamingResponse:
    def __init__(self, whatsapp: AsyncWhatsappResource) -> None:
        self._whatsapp = whatsapp

    @cached_property
    def business_accounts(self) -> AsyncBusinessAccountsResourceWithStreamingResponse:
        """Manage Whatsapp business accounts"""
        return AsyncBusinessAccountsResourceWithStreamingResponse(self._whatsapp.business_accounts)

    @cached_property
    def templates(self) -> AsyncTemplatesResourceWithStreamingResponse:
        """Manage Whatsapp message templates"""
        return AsyncTemplatesResourceWithStreamingResponse(self._whatsapp.templates)

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncPhoneNumbersResourceWithStreamingResponse(self._whatsapp.phone_numbers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["RegulatoryRequirementRetrieveParams", "Filter"]


class RegulatoryRequirementRetrieveParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number], filter[requirement_group_id],
    filter[country_code], filter[phone_number_type], filter[action]
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number], filter[requirement_group_id], filter[country_code], filter[phone_number_type], filter[action]
    """

    action: Literal["ordering", "porting", "action"]
    """Action to check requirements for"""

    country_code: str
    """Country code(iso2) to check requirements for"""

    phone_number: str
    """Phone number to check requirements for"""

    phone_number_type: Literal["local", "toll_free", "mobile", "national", "shared_cost"]
    """Phone number type to check requirements for"""

    requirement_group_id: str
    """ID of requirement group to check requirements for"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .available_service import AvailableService
from .shared.netapps_location import NetappsLocation

__all__ = ["NetworkCoverageListResponse"]


class NetworkCoverageListResponse(BaseModel):
    available_services: Optional[List[AvailableService]] = None
    """List of interface types supported in this region."""

    location: Optional[NetappsLocation] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .action_check_registration_status_response import (
    ActionCheckRegistrationStatusResponse as ActionCheckRegistrationStatusResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .verification.verification import (
    VerificationResource,
    AsyncVerificationResource,
    VerificationResourceWithRawResponse,
    AsyncVerificationResourceWithRawResponse,
    VerificationResourceWithStreamingResponse,
    AsyncVerificationResourceWithStreamingResponse,
)

__all__ = ["MessagingTollfreeResource", "AsyncMessagingTollfreeResource"]


class MessagingTollfreeResource(SyncAPIResource):
    @cached_property
    def verification(self) -> VerificationResource:
        return VerificationResource(self._client)

    @cached_property
    def with_raw_response(self) -> MessagingTollfreeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagingTollfreeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagingTollfreeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagingTollfreeResourceWithStreamingResponse(self)


class AsyncMessagingTollfreeResource(AsyncAPIResource):
    @cached_property
    def verification(self) -> AsyncVerificationResource:
        return AsyncVerificationResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMessagingTollfreeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagingTollfreeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagingTollfreeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagingTollfreeResourceWithStreamingResponse(self)


class MessagingTollfreeResourceWithRawResponse:
    def __init__(self, messaging_tollfree: MessagingTollfreeResource) -> None:
        self._messaging_tollfree = messaging_tollfree

    @cached_property
    def verification(self) -> VerificationResourceWithRawResponse:
        return VerificationResourceWithRawResponse(self._messaging_tollfree.verification)


class AsyncMessagingTollfreeResourceWithRawResponse:
    def __init__(self, messaging_tollfree: AsyncMessagingTollfreeResource) -> None:
        self._messaging_tollfree = messaging_tollfree

    @cached_property
    def verification(self) -> AsyncVerificationResourceWithRawResponse:
        return AsyncVerificationResourceWithRawResponse(self._messaging_tollfree.verification)


class MessagingTollfreeResourceWithStreamingResponse:
    def __init__(self, messaging_tollfree: MessagingTollfreeResource) -> None:
        self._messaging_tollfree = messaging_tollfree

    @cached_property
    def verification(self) -> VerificationResourceWithStreamingResponse:
        return VerificationResourceWithStreamingResponse(self._messaging_tollfree.verification)


class AsyncMessagingTollfreeResourceWithStreamingResponse:
    def __init__(self, messaging_tollfree: AsyncMessagingTollfreeResource) -> None:
        self._messaging_tollfree = messaging_tollfree

    @cached_property
    def verification(self) -> AsyncVerificationResourceWithStreamingResponse:
        return AsyncVerificationResourceWithStreamingResponse(self._messaging_tollfree.verification)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["PortingOrderUserFeedback"]


class PortingOrderUserFeedback(BaseModel):
    user_comment: Optional[str] = None
    """A comment related to the customer rating."""

    user_rating: Optional[int] = None
    """
    Once an order is ported, cancellation is requested or the request is cancelled,
    the user may rate their experience
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.whatsapp import template_list_params, template_create_params
from ...types.shared.whatsapp_template_data import WhatsappTemplateData
from ...types.whatsapp.template_create_response import TemplateCreateResponse

__all__ = ["TemplatesResource", "AsyncTemplatesResource"]


class TemplatesResource(SyncAPIResource):
    """Manage Whatsapp message templates"""

    @cached_property
    def with_raw_response(self) -> TemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TemplatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"],
        components: Iterable[object],
        language: str,
        name: str,
        waba_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TemplateCreateResponse:
        """
        Create a Whatsapp message template

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/whatsapp/message_templates",
            body=maybe_transform(
                {
                    "category": category,
                    "components": components,
                    "language": language,
                    "name": name,
                    "waba_id": waba_id,
                },
                template_create_params.TemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TemplateCreateResponse,
        )

    def list(
        self,
        *,
        filter_category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"] | Omit = omit,
        filter_search: str | Omit = omit,
        filter_status: str | Omit = omit,
        filter_waba_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[WhatsappTemplateData]:
        """
        List Whatsapp message templates

        Args:
          filter_category: Filter by category

          filter_search: Search templates by name

          filter_status: Filter by template status

          filter_waba_id: Filter by WABA ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/whatsapp/message_templates",
            page=SyncDefaultFlatPagination[WhatsappTemplateData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_category": filter_category,
                        "filter_search": filter_search,
                        "filter_status": filter_status,
                        "filter_waba_id": filter_waba_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    template_list_params.TemplateListParams,
                ),
            ),
            model=WhatsappTemplateData,
        )


class AsyncTemplatesResource(AsyncAPIResource):
    """Manage Whatsapp message templates"""

    @cached_property
    def with_raw_response(self) -> AsyncTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTemplatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"],
        components: Iterable[object],
        language: str,
        name: str,
        waba_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TemplateCreateResponse:
        """
        Create a Whatsapp message template

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/whatsapp/message_templates",
            body=await async_maybe_transform(
                {
                    "category": category,
                    "components": components,
                    "language": language,
                    "name": name,
                    "waba_id": waba_id,
                },
                template_create_params.TemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TemplateCreateResponse,
        )

    def list(
        self,
        *,
        filter_category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"] | Omit = omit,
        filter_search: str | Omit = omit,
        filter_status: str | Omit = omit,
        filter_waba_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WhatsappTemplateData, AsyncDefaultFlatPagination[WhatsappTemplateData]]:
        """
        List Whatsapp message templates

        Args:
          filter_category: Filter by category

          filter_search: Search templates by name

          filter_status: Filter by template status

          filter_waba_id: Filter by WABA ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/whatsapp/message_templates",
            page=AsyncDefaultFlatPagination[WhatsappTemplateData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_category": filter_category,
                        "filter_search": filter_search,
                        "filter_status": filter_status,
                        "filter_waba_id": filter_waba_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    template_list_params.TemplateListParams,
                ),
            ),
            model=WhatsappTemplateData,
        )


class TemplatesResourceWithRawResponse:
    def __init__(self, templates: TemplatesResource) -> None:
        self._templates = templates

        self.create = to_raw_response_wrapper(
            templates.create,
        )
        self.list = to_raw_response_wrapper(
            templates.list,
        )


class AsyncTemplatesResourceWithRawResponse:
    def __init__(self, templates: AsyncTemplatesResource) -> None:
        self._templates = templates

        self.create = async_to_raw_response_wrapper(
            templates.create,
        )
        self.list = async_to_raw_response_wrapper(
            templates.list,
        )


class TemplatesResourceWithStreamingResponse:
    def __init__(self, templates: TemplatesResource) -> None:
        self._templates = templates

        self.create = to_streamed_response_wrapper(
            templates.create,
        )
        self.list = to_streamed_response_wrapper(
            templates.list,
        )


class AsyncTemplatesResourceWithStreamingResponse:
    def __init__(self, templates: AsyncTemplatesResource) -> None:
        self._templates = templates

        self.create = async_to_streamed_response_wrapper(
            templates.create,
        )
        self.list = async_to_streamed_response_wrapper(
            templates.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["PlanUpdateStepParams"]


class PlanUpdateStepParams(TypedDict, total=False):
    mission_id: Required[str]

    run_id: Required[str]

    metadata: Dict[str, object]

    status: Literal["pending", "in_progress", "completed", "skipped", "failed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import sim_card_order_preview_preview_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.sim_card_order_preview_preview_response import SimCardOrderPreviewPreviewResponse

__all__ = ["SimCardOrderPreviewResource", "AsyncSimCardOrderPreviewResource"]


class SimCardOrderPreviewResource(SyncAPIResource):
    """SIM Card Orders operations"""

    @cached_property
    def with_raw_response(self) -> SimCardOrderPreviewResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SimCardOrderPreviewResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SimCardOrderPreviewResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SimCardOrderPreviewResourceWithStreamingResponse(self)

    def preview(
        self,
        *,
        address_id: str,
        quantity: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardOrderPreviewPreviewResponse:
        """
        Preview SIM card order purchases.

        Args:
          address_id: Uniquely identifies the address for the order.

          quantity: The amount of SIM cards that the user would like to purchase in the SIM card
              order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/sim_card_order_preview",
            body=maybe_transform(
                {
                    "address_id": address_id,
                    "quantity": quantity,
                },
                sim_card_order_preview_preview_params.SimCardOrderPreviewPreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardOrderPreviewPreviewResponse,
        )


class AsyncSimCardOrderPreviewResource(AsyncAPIResource):
    """SIM Card Orders operations"""

    @cached_property
    def with_raw_response(self) -> AsyncSimCardOrderPreviewResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSimCardOrderPreviewResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSimCardOrderPreviewResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSimCardOrderPreviewResourceWithStreamingResponse(self)

    async def preview(
        self,
        *,
        address_id: str,
        quantity: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardOrderPreviewPreviewResponse:
        """
        Preview SIM card order purchases.

        Args:
          address_id: Uniquely identifies the address for the order.

          quantity: The amount of SIM cards that the user would like to purchase in the SIM card
              order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/sim_card_order_preview",
            body=await async_maybe_transform(
                {
                    "address_id": address_id,
                    "quantity": quantity,
                },
                sim_card_order_preview_preview_params.SimCardOrderPreviewPreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardOrderPreviewPreviewResponse,
        )


class SimCardOrderPreviewResourceWithRawResponse:
    def __init__(self, sim_card_order_preview: SimCardOrderPreviewResource) -> None:
        self._sim_card_order_preview = sim_card_order_preview

        self.preview = to_raw_response_wrapper(
            sim_card_order_preview.preview,
        )


class AsyncSimCardOrderPreviewResourceWithRawResponse:
    def __init__(self, sim_card_order_preview: AsyncSimCardOrderPreviewResource) -> None:
        self._sim_card_order_preview = sim_card_order_preview

        self.preview = async_to_raw_response_wrapper(
            sim_card_order_preview.preview,
        )


class SimCardOrderPreviewResourceWithStreamingResponse:
    def __init__(self, sim_card_order_preview: SimCardOrderPreviewResource) -> None:
        self._sim_card_order_preview = sim_card_order_preview

        self.preview = to_streamed_response_wrapper(
            sim_card_order_preview.preview,
        )


class AsyncSimCardOrderPreviewResourceWithStreamingResponse:
    def __init__(self, sim_card_order_preview: AsyncSimCardOrderPreviewResource) -> None:
        self._sim_card_order_preview = sim_card_order_preview

        self.preview = async_to_streamed_response_wrapper(
            sim_card_order_preview.preview,
        )
